﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using HeuristicLab.DataImporter.Data.CommandBase;
using HeuristicLab.DataImporter.Data.Model;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataImporter.Command {
  [StorableClass]
  public abstract class AggregateCommandBase : ColumnGroupCommandWithAffectedColumnsBase {
    private AggregateCommandBase()
      : base(null, string.Empty, null) {
      oldColumns = new List<ColumnBase>();
      newColumns = new List<ColumnBase>();
      positions = new List<int>();
      changeNullValuesOnly = true;
    }

    protected AggregateCommandBase(DataSet dataSet, string columnGroupName, int[] affectedColumns)
      : base(dataSet, columnGroupName, affectedColumns) {
      oldColumns = new List<ColumnBase>();
      newColumns = new List<ColumnBase>();
      positions = new List<int>();
      changeNullValuesOnly = true;
    }

    private List<ColumnBase> oldColumns;
    protected List<ColumnBase> OldColumns {
      get { return this.oldColumns; }
    }

    private List<ColumnBase> newColumns;
    protected List<ColumnBase> NewColumns {
      get { return this.newColumns; }
    }

    private List<int> positions;
    protected List<int> Positions {
      get { return this.positions; }
    }

    [Storable]
    private bool changeNullValuesOnly;
    public bool ChangeNullValueOnly {
      get { return this.changeNullValuesOnly; }
      set { this.changeNullValuesOnly = value; }
    }

    protected abstract IComparable AggregationFunction(ColumnBase col, int startIndex, int endIndex);

    public override void Execute() {
      base.Execute();
      if (!ColumnGroup.Sorted)
        throw new CommandExecutionException("ColumnGroup must be sorted to use aggregation commands.", this);
      ColumnBase col;
      for (int i = 0; i < AffectedColumns.Length; i++) {
        col = ColumnGroup.Columns.ElementAt(AffectedColumns[i]);
        if (col.DataType == typeof(double?) && !ColumnGroup.SortedColumnIndexes.Contains(AffectedColumns[i])) {
          OldColumns.Add(col);
          NewColumns.Add(col.CreateCopyOfColumnWithoutValues());
          Positions.Add(AffectedColumns[i]);
        }
      }
      IComparable keyValue = ColumnGroup.KeyString(0);
      IComparable aggregatedValue;
      int startIndex = 0;
      int endIndex = -1;
      for (int i = 1; i <= ColumnGroup.RowCount; i++) {
        if (i == ColumnGroup.RowCount || keyValue.CompareTo(ColumnGroup.KeyString(i)) != 0) {
          endIndex = i;
          for (int colIndex = 0; colIndex < OldColumns.Count; colIndex++) {
            aggregatedValue = AggregationFunction(OldColumns[colIndex], startIndex, endIndex);
            //add aggregatedValue for each row with the same keyColumnValue
            for (int j = startIndex; j < endIndex; j++) {
              if (!changeNullValuesOnly || OldColumns[colIndex].GetValue(j) == null)
                NewColumns[colIndex].AddValue(aggregatedValue);
              else
                NewColumns[colIndex].AddValue(OldColumns[colIndex].GetValue(j));
            }
          }
          if (i != ColumnGroup.RowCount) {
            startIndex = i;
            keyValue = ColumnGroup.KeyString(i);
          }
        }
      }

      for (int i = 0; i < Positions.Count; i++) {
        ColumnGroup.ReplaceColumn(Positions[i], NewColumns[i]);
        OldColumns[i].Selected = false;
      }
      NewColumns.Clear();
      ColumnGroup.FireChanged();
      ColumnGroup = null;
    }

    public override void UndoExecute() {
      base.UndoExecute();
      for (int i = 0; i < Positions.Count; i++) {
        ColumnGroup.ReplaceColumn(Positions[i], OldColumns[i]);
      }
      OldColumns.Clear();
      Positions.Clear();
      ColumnGroup.FireChanged();
      ColumnGroup = null;
    }
  }
}
