﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.DataImporter.Data;
using HeuristicLab.DataImporter.Data.CommandBase;
using HeuristicLab.DataImporter.Data.Model;
using HeuristicLab.DataImporter.Command.View;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataImporter.Command {
  [StorableClass]
  [ViewableCommandInfoAttribute("Minimum", 1, ColumnGroupState.DoubleColumnSelected | ColumnGroupState.AnySelectedColumnNotSorted,
   "Aggregation", Position = 3, OptionsView = typeof(AggregationCommandView))]
  public class AggregateWithMinCommand : AggregateCommandBase {

    private AggregateWithMinCommand()
      : base(null, string.Empty, null) {
    }

    public AggregateWithMinCommand(DataSet dataSet, string columnGroupName, int[] affectedColumns)
      : base(dataSet, columnGroupName, affectedColumns) {
    }

    public override string Description {
      get { return "Aggregate with min function"; }
    }

    protected override IComparable AggregationFunction(ColumnBase col, int startIndex, int endIndex) {
      IComparable minValue = null;
      IComparable val;
      for (int i = startIndex; i < endIndex; i++) {
        val = col.GetValue(i);
        if (val != null) {
          if (minValue == null || val.CompareTo(minValue) < 0)
            minValue = val;
        }
      }
      return minValue;
    }
  }
}
