﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.DataImporter.Data;
using HeuristicLab.DataImporter.Data.CommandBase;
using HeuristicLab.DataImporter.Data.Model;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataImporter.Command {
  [StorableClass]
  [ViewableCommandInfoAttribute("Delete Rows", 1, ColumnGroupState.Sorted, "Aggregation", Position = 5)]
  public class DeleteRowsWithDuplicateKeyValuesCommand : ColumnGroupCommandBase {
    private Dictionary<int, IComparable[]> deletedRows;
    private DeleteRowsWithDuplicateKeyValuesCommand()
      : base(null, string.Empty) {
      deletedRows = new Dictionary<int, IComparable[]>();
    }

    public DeleteRowsWithDuplicateKeyValuesCommand(DataSet dataSet, string columnGroupName)
      : base(dataSet, columnGroupName) {
      deletedRows = new Dictionary<int, IComparable[]>();
    }

    public override string Description {
      get { return "Delete rows with duplicate key values"; }
    }

    public override void Execute() {
      base.Execute();
      if (!ColumnGroup.Sorted)
        throw new CommandExecutionException("ColumnGroup must be sorted to delete rows with duplicate key.", this);

      for (int i = 0; i < ColumnGroup.RowCount - 1; i++) {
        if (ColumnGroup.KeyString(i) == ColumnGroup.KeyString(i + 1))
          deletedRows[i + 1] = ColumnGroup.GetRow(i + 1);
      }

      foreach (int i in deletedRows.Keys.Reverse())
        ColumnGroup.DeleteRow(i);

      ColumnGroup.FireChanged();
    }

    public override void UndoExecute() {
      base.UndoExecute();
      foreach (KeyValuePair<int, IComparable[]> deletedRow in deletedRows)
        ColumnGroup.InsertRow(deletedRow.Key, deletedRow.Value);
      deletedRows.Clear();
      ColumnGroup.FireChanged();

    }
  }
}
